import copy

from screen import screen_size


W, H = screen_size


class OffsetEvent(object):
    def __init__(self, x, y):
        self.x = x
        self.y = y

    @property
    def xy(self):
        return self.x, self.y

    def create_offset_event(self, ox, oy):
        x, y = self.pixel_coords()

        offset_event = copy.copy(self)
        offset_event.x = x - ox
        offset_event.y = y - oy
        offset_event.original_event = self

        return offset_event

    def pixel_coords(self):
        return self.x * W, self.y * H


class Event(OffsetEvent):
    def __init__(self, point):
        OffsetEvent.__init__(self, point.x, point.y)
        self.point = point

    def __str__(self):
        return '<%s (%s, %s) sid=%d>' % \
               (self.__class__.__name__, self.x, self.y, self.point.sid)

    def __getattr__(self, name):
        """
        Delegate attributes to touch point.
        """
        if hasattr(self.point, name):
            return getattr(self.point, name)

        raise AttributeError


class Gesture(OffsetEvent):
    def __str__(self):
        return '<%s (%s, %s)>' % (self.__class__.__name__, self.x, self.y)


class DownEvent(Event):
    _name = 'down'


class UpEvent(Event):
    _name = 'up'


class MoveEvent(Event):
    _name = 'move'

    def __str__(self):
        return '<%s (%s, %s) to (%s, %s) sid=%d>' % \
               (self.__class__.__name__, self.point.px, self.point.py,
                self.point.x, self.point.y, self.point.sid)


class Tap(Gesture):
    _name = 'tap'


class SingleTap(Gesture):
    _name = 'single_tap'


class DoubleTap(Gesture):
    _name = 'double_tap'


class Flick(Gesture):
    _name = 'flick'

    def __init__(self, x, y, velocity):
        super(Flick, self).__init__(x, y)
        self.velocity = velocity

    def __str__(self):
        return '<%s (%s, %s) velocity=%s>' % \
               (self.__class__.__name__, self.x, self.y, self.velocity)


class Rotate(Gesture):
    _name = 'rotate'

    def __init__(self, cx, cy, angle, n):
        super(Rotate, self).__init__(cx, cy)
        self.angle = angle
        self.n = n

    def __str__(self):
        return '<%s (%s, %s) angle=%s n=%d>' % \
               (self.__class__.__name__, self.x, self.y, self.angle, self.n)


class Pinch(Gesture):
    _name = 'pinch'

    def __init__(self, cx, cy, amount, n):
        super(Pinch, self).__init__(cx, cy)
        self.amount = amount
        self.n = n

    def __str__(self):
        return '<%s (%s, %s) amount=%s n=%d>' % \
               (self.__class__.__name__, self.x, self.y, self.amount, self.n)


class Pan(Gesture):
    _name = 'pan'

    def __init__(self, x, y, dx, dy, n):
        super(Pan, self).__init__(x, y)
        self.dx = dx
        self.dy = dy
        self.n = n

    def __str__(self):
        return '<%s (%s, %s) direction=(%s, %s) n=%d>' % \
               (self.__class__.__name__, self.x, self.y, self.dx, self.dy,
                self.n)
